/**
    \file 
        TerrainContentGenerator.cpp
    \brief
        Creates content for the terrain.
*/
#include "TerrainContentGenerator.h"
#include <windows.h>
/********************************************/
/*  TerrainContentGenerator implementation  */
/********************************************/
//-----------------------------------------------------------------------
TerrainContentGenerator::TerrainContentGenerator(SceneManager *sm)
{
	mSceneMgr = sm;
	mRayQueryExecutor = new RayQueryExecutor(sm);

	mMinPos = Vector3(-500.0f, 5000.0f, -500.0f);
	mMaxPos = Vector3(500.0f, 5000.0f, 500.0f);

	mMinAngle = 0;
	mMaxAngle = 360;

	mCount = 0;
}
//-----------------------------------------------------------------------
TerrainContentGenerator::~TerrainContentGenerator()
{
	delete mRayQueryExecutor;
}
//-----------------------------------------------------------------------
void TerrainContentGenerator::generateScene(int numObjects, const String &meshName)
{
	srand (time(0));
	
	Vector3 rotationRatio;
	Vector3 translationRatio;

	for(int i=0; i < numObjects; ++i)
	{
		rotationRatio.x = rand() / (float) RAND_MAX;
		rotationRatio.y = rand() / (float) RAND_MAX;
		rotationRatio.z = rand() / (float) RAND_MAX;

		translationRatio.x = rand() / (float) RAND_MAX;
		translationRatio.y = rand() / (float) RAND_MAX;
		translationRatio.z = rand() / (float) RAND_MAX;

		generateSceneObject(translationRatio, rotationRatio, i, meshName);
	}
}
//-----------------------------------------------------------------------
void TerrainContentGenerator::generateSceneObject(const Vector3 &translationRatio, 
					 const Vector3 &rotationRatio, const int idx, const String& entName)
{
	// Setup the ray scene query
	Real rotation = mMinAngle + rotationRatio.x * (mMaxAngle - mMinAngle);
	Vector3 position = mMinPos + translationRatio * (mMaxPos - mMinPos);

	Vector3 queryResult;

	char msg[100];
	sprintf(msg, "pos: %3.3f %3.3f %3.3f\n", position.x, position.y, position.z);

	if (mRayQueryExecutor->executeRayQuery(&queryResult, position, Vector3::NEGATIVE_UNIT_Y))
	{
		char name[16];
		sprintf(name, "%s%d", entName.c_str(), mCount++);
		OutputDebugString("Creating new object "); OutputDebugString(msg);

		Entity *ent = mSceneMgr->createEntity(name, entName);
		SceneNode *currentObject = mSceneMgr->getRootSceneNode()->
			createChildSceneNode(String(name) + "Node", queryResult);

		currentObject->attachObject(ent);
		currentObject->setScale(0.1f, 0.1f, 0.1f);
		currentObject->setPosition(position);
	}
	else
	{
		OutputDebugString(msg);
	}
}
/****************************************/
/*  RayQueryExecutor implementation     */
/****************************************/
//-----------------------------------------------------------------------
RayQueryExecutor::RayQueryExecutor(SceneManager *sm)
{
	mRaySceneQuery = sm->createRayQuery(Ray());
}
//-----------------------------------------------------------------------
RayQueryExecutor::~RayQueryExecutor()
{
	delete mRaySceneQuery;
}
//-----------------------------------------------------------------------
bool RayQueryExecutor::executeRayQuery(Vector3 *result, const Vector3 &pos, const Vector3 &dir)
{
	return executeRayQuery(result, Ray(pos, dir));
}

bool RayQueryExecutor::executeRayQuery(Vector3 *result, const Ray &ray)
{
    mRaySceneQuery->setRay(ray);

    // Perform the scene query
    RaySceneQueryResult &queryResult = mRaySceneQuery->execute();
    RaySceneQueryResult::iterator it = queryResult.begin();

	if (it != queryResult.end() && it->worldFragment)
	{
		SceneQuery::WorldFragment* wf = it->worldFragment;

		*result = wf->singleIntersection; 
		return true;
	}

	return false;
}
